@@ -23,6 +23,7 @@
 #include <KWayland/Client/registry.h>
 #include <KWayland/Client/slide.h>
 #include <KWayland/Client/surface.h>
+#include <private/qwaylandwindow_p.h>
 
 WindowEffects::WindowEffects()
     : QObject()
@@ -89,24 +90,58 @@ void WindowEffects::trackWindow(QWindow *window)
     if (!m_windowWatchers.contains(window)) {
         window->installEventFilter(this);
         auto conn = connect(window, &QObject::destroyed, this, [this, window]() {
+            resetBlur(window);
             m_blurRegions.remove(window);
+            resetContrast(window);
             m_backgroundConstrastRegions.remove(window);
             m_slideMap.remove(window);
             m_windowWatchers.remove(window);
         });
-        m_windowWatchers[window] = conn;
+        m_windowWatchers[window] << conn;
+        auto waylandWindow = dynamic_cast<QtWaylandClient::QWaylandWindow *>(window->handle());
+        if (waylandWindow) {
+            auto conn = connect(waylandWindow, &QtWaylandClient::QWaylandWindow::wlSurfaceDestroyed, this, [this, window]() {
+                resetBlur(window);
+                resetContrast(window);
+            });
+            m_windowWatchers[window] << conn;
+        }
     }
 }
 
 void WindowEffects::releaseWindow(QWindow *window)
 {
     if (!m_blurRegions.contains(window) && !m_backgroundConstrastRegions.contains(window) && !m_slideMap.contains(window)) {
-        disconnect(m_windowWatchers[window]);
+        for (const auto &conn : m_windowWatchers[window]) {
+            disconnect(conn);
+        }
         window->removeEventFilter(this);
         m_windowWatchers.remove(window);
     }
 }
 
+// Helper function to replace a QObject value in the map and delete the old one.
+template<typename MapType>
+void replaceValue(MapType &map, typename MapType::key_type key, typename MapType::mapped_type value)
+{
+    if (auto oldValue = map.take(key)) {
+        oldValue->deleteLater();
+    }
+    if (value) {
+        map[key] = value;
+    }
+}
+
+void WindowEffects::resetBlur(QWindow *window, KWayland::Client::Blur *blur)
+{
+    replaceValue(m_blurs, window, blur);
+}
+
+void WindowEffects::resetContrast(QWindow *window, KWayland::Client::Contrast *contrast)
+{
+    replaceValue(m_contrasts, window, contrast);
+}
+
 bool WindowEffects::eventFilter(QObject *watched, QEvent *event)
 {
     if (event->type() == QEvent::Expose) {
@@ -252,6 +287,7 @@ void WindowEffects::enableBlurBehind(WId winId, bool enable, const QRegion &regi
         trackWindow(window);
         m_blurRegions[window] = region;
     } else {
+        resetBlur(window);
         m_blurRegions.remove(window);
         releaseWindow(window);
     }
@@ -269,9 +305,12 @@ void WindowEffects::installBlur(QWindow *window, bool enable, const QRegion &reg
     if (surface) {
         if (enable) {
             auto blur = WaylandIntegration::self()->waylandBlurManager()->createBlur(surface, surface);
-            blur->setRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+            std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+            blur->setRegion(wlRegion.get());
             blur->commit();
+            resetBlur(window, blur);
         } else {
+            resetBlur(window);
             WaylandIntegration::self()->waylandBlurManager()->removeBlur(surface);
         }
 
@@ -292,6 +331,7 @@ void WindowEffects::enableBackgroundContrast(WId winId, bool enable, qreal contr
         m_backgroundConstrastRegions[window].saturation = saturation;
         m_backgroundConstrastRegions[window].region = region;
     } else {
+        resetContrast(window);
         m_backgroundConstrastRegions.remove(window);
         releaseWindow(window);
     }
@@ -308,12 +348,15 @@ void WindowEffects::installContrast(QWindow *window, bool enable, qreal contrast
     if (surface) {
         if (enable) {
             auto backgroundContrast = WaylandIntegration::self()->waylandContrastManager()->createContrast(surface, surface);
-            backgroundContrast->setRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+            std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+            backgroundContrast->setRegion(wlRegion.get());
             backgroundContrast->setContrast(contrast);
             backgroundContrast->setIntensity(intensity);
             backgroundContrast->setSaturation(saturation);
             backgroundContrast->commit();
+            resetContrast(window, backgroundContrast);
         } else {
+            resetContrast(window);
             WaylandIntegration::self()->waylandContrastManager()->removeContrast(surface);
         }
 
@@ -332,14 +375,17 @@ void WindowEffects::setBackgroundFrost(QWindow *window, QColor color, const QReg
         return;
     }
     if (!color.isValid()) {
+        resetContrast(window);
         WaylandIntegration::self()->waylandContrastManager()->removeContrast(surface);
         return;
     }
 
     auto backgroundContrast = WaylandIntegration::self()->waylandContrastManager()->createContrast(surface, surface);
-    backgroundContrast->setRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+    std::unique_ptr<KWayland::Client::Region> wlRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+    backgroundContrast->setRegion(wlRegion.get());
     backgroundContrast->setFrost(color);
     backgroundContrast->commit();
+    resetContrast(window, backgroundContrast);
 
     WaylandIntegration::self()->waylandConnection()->flush();
 }
