@@ -346,7 +346,7 @@ void WindowSystem::clearState(WId win, NET::States state)
 
     KWayland::Client::PlasmaShellSurface *plasmaShellSurface = nullptr;
 
-    if (state & NET::SkipTaskbar) {
+    if (state & NET::SkipTaskbar || state & NET::SkipSwitcher) {
         if (!WaylandIntegration::self()->waylandPlasmaShell()) {
             return;
         }
@@ -363,6 +363,10 @@ void WindowSystem::clearState(WId win, NET::States state)
         plasmaShellSurface->setSkipTaskbar(false);
     }
 
+    if (state & NET::SkipSwitcher) {
+        plasmaShellSurface->setSkipSwitcher(false);
+    }
+
     if (state & NET::Max) {
         qCDebug(KWAYLAND_KWS) << "This plugin does not support changing Max window state";
     }
@@ -407,7 +411,7 @@ void WindowSystem::setState(WId win, NET::States state)
 
     KWayland::Client::PlasmaShellSurface *plasmaShellSurface = nullptr;
 
-    if (state & NET::SkipTaskbar) {
+    if (state & NET::SkipTaskbar || state & NET::SkipSwitcher) {
         if (!WaylandIntegration::self()->waylandPlasmaShell()) {
             return;
         }
@@ -424,6 +428,10 @@ void WindowSystem::setState(WId win, NET::States state)
         plasmaShellSurface->setSkipTaskbar(true);
     }
 
+    if (state & NET::SkipSwitcher) {
+        plasmaShellSurface->setSkipSwitcher(true);
+    }
+
     if (state & NET::Max) {
         qCDebug(KWAYLAND_KWS) << "This plugin does not support changing Max window state";
     }
